//
// This code is part of https://developer.mescius.com/document-solutions/dot-net-word-api/demos.
// Copyright (c) MESCIUS inc. All rights reserved.
//
using System;
using System.IO;
using System.Drawing;
using System.Collections.Generic;
using System.Linq;
using System.Globalization;
using GrapeCity.Documents.Word;

namespace DsWordWeb.Demos
{
    // This example shows how to deal with the 'data source not found' error.
    public class DataTplFixDataSourceNotFound
    {
        // Code demonstrating the problem:
        GcWordDocument Problem()
        {
            using var oceans = File.OpenRead(Path.Combine("Resources", "data", "oceans.json"));
            var doc = new GcWordDocument();
            doc.DataTemplate.DataSources.Add("ds", oceans);
            // Incorrect: 'ds1' is used as data source name, but there is no data source with that name:
            doc.Body.Paragraphs.Add("{{ds1.name}}");
            doc.DataTemplate.Process(CultureInfo.GetCultureInfo("en-US"));
            return doc;
        }

        // Code demonstrating the fix:
        GcWordDocument Fix()
        {
            using var oceans = File.OpenRead(Path.Combine("Resources", "data", "oceans.json"));
            var doc = new GcWordDocument();
            doc.DataTemplate.DataSources.Add("ds", oceans);
            // Correct: use the correct data source name 'ds':
            doc.Body.Paragraphs.Add("{{ds.name}}");
            doc.DataTemplate.Process(CultureInfo.GetCultureInfo("en-US"));
            return doc;
        }

        public GcWordDocument CreateDocx()
        {
            GcWordDocument doc;
            try
            {
                // This fails:
                doc = Problem();
            }
            catch (Exception ex)
            {
                // This works:
                doc = Fix();
                // Insert a brief explanation of the problem and the fix into the generated document:
                doc.Body.Paragraphs.Insert(
                    $"The error \"{ex.Message}\" occurred because in the template a non-existing data source name was used. " +
                    $"The fix is to ensure that data source names used in the template match names of data sources added to the DataSources collection.",
                    doc.Styles[BuiltInStyleId.BlockText],
                    InsertLocation.Start);
            }
            return doc;
        }
    }
}
